# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013
#
# Author: Daniel Chapman daniel@chapman-mail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Contains, FileExists, \
    FileContains
from autopilot.input import Pointer, Mouse
import tempfile
import os


class GeditAutopilotTestCase(AutopilotTestCase):

    def setUp(self):
        super(GeditAutopilotTestCase, self).setUp()
        self.app = self.launch_test_application("gedit")
        self.pointing_device = Pointer(Mouse.create())

    def setup_test_docs(self, num_of_docs, text=False,
                        tab_num=1, save_dlg=False,
                        opn_dlg=False, replace_dlg=False,
                        print_dlg=False, print_preview=False):
        """
        This function sets up the pre-reqs for a test

        Using a combination of these params you can set up different
        situations for testing

        params:
        num_of_docs: *Required* an int value of the number of docs
                     needed for the test
        text=: Set to False by default if set to true will add text
               to each document.
        tab_num=: Sets the tab focus to the given tab number on setup.
                  Set to 1 by default
        save_dlg=: If set to True Sets the save dialog up in the
                   gedit/samples directory for the test
        opn_dlg=: If set to True sets the open dialog up for test
        print_dlg=: If set to True sets the print
                    dialog up for printing to a file
        print_preview=: If set to True sets the Print preview
                        dialog up for testing
        """
        # gedit by default loads a blank document
        # so start num at 1
        num = 1
        txt = text

        if txt is True:
            view = self.app.select_single('GeditView')
            self.assertThat(view.is_focus, Equals(1))
            self.keyboard.type(
                'Document %s: The quick brown fox jumps over the lazy dog'
                % str(num))

            self.assertThat(
                view.buffer,
                Equals(
                    'Document %s: The quick brown fox jumps over the lazy dog'
                    % str(num)),
                "Autopilot mis-typed the phrase")

        while num < num_of_docs:
            self.keyboard.press_and_release('Ctrl+n')
            num += 1
            if txt is True:
                view = self.app.select_many('GeditView')[num - 1]
                self.assertThat(view.is_focus, Equals(1))
                self.keyboard.type(
                    'Document %s: The quick brown fox jumps over the lazy dog'
                    % str(num))

                self.assertThat(
                    view.buffer,
                    Equals(
                        'Document %s: The quick brown fox jumps \
                        over the lazy dog' % str(num)),
                    "Autopilot mis-typed the phrase")

        # lets just check we have the right amount of tabs
        num_of_tabs = len(self.app.select_many('GeditView'))
        self.assertTrue(num_of_tabs, Equals(num_of_docs))

        # focus the required tab window
        tab = str(tab_num)
        self.keyboard.press_and_release('Alt+%s' % tab)
        # check we are on the correct tab
        view = self.app.select_many('GeditView')[tab_num - 1]
        self.assertThat(view.is_focus, Eventually(Equals(1)),
                        "We are not on the correct tab")
        if save_dlg is True:
            self.pointing_device.click_object(
                self.app.select_single('GtkToolButton',
                                       name='FileSave'))
            self.assertThat(
                lambda: self.app.select_single(
                    'GeditFileChooserDialog').visible,
                Eventually(Equals(1)), "The save dialog did not open")
            dlg = self.app.select_single('GeditFileChooserDialog')

            self.pointing_device.click_object(
                dlg.select_single('GtkFileChooserEntry'))
            self.assertThat(
                lambda: dlg.select_single('GtkFileChooserEntry').is_focus,
                Eventually(Equals(1)),
                "Entry did not get focus after being clicked")
            self.keyboard.press_and_release('Ctrl+a')
            self.keyboard.press_and_release('Delete')
            # lets make sure we always save in /tmp
            self.keyboard.type('/tmp/')
            self.keyboard.press_and_release('Enter')

        if opn_dlg is True:
            self.pointing_device.click_object(
                self.app.select_single('GeditOpenToolButton'))
            self.assertThat(
                lambda: self.app.select_single(
                    'GeditFileChooserDialog').visible,
                Eventually(Equals(1)), "file chooser dialog did not open")
            dlg = self.app.select_single('GeditFileChooserDialog')
            self.keyboard.press_and_release('/')
            self.assertThat(
                lambda: dlg.select_single('GtkFileChooserEntry').is_focus,
                Eventually(Equals(1)),
                "Entry did not get focus after being clicked")
            # lets make sure we are in our samples dir
            sample_dir = 'samples'
            for root, dirs, files in os.walk(os.getcwd()):
                for name in dirs:
                    if name == sample_dir:
                        dir_path = os.path.abspath(os.path.join(root, name))
            self.keyboard.type(dir_path + "/")

        if replace_dlg is True:
            self.pointing_device.click_object(
                self.app.select_single('GtkToolButton',
                                       name='SearchReplace'))
            self.assertThat(
                lambda: self.app.select_single('GeditReplaceDialog').visible,
                Eventually(Equals(1)),
                "Replace dialog did not open")
            # lets grab all the objects we might need in the test
            self.rep_dialog = self.app.select_single('GeditReplaceDialog')
            self.mtch_chkbtn = self.rep_dialog.select_single(
                BuilderName='match_case_checkbutton')
            self.entire_chkbtn = self.rep_dialog.select_single(
                BuilderName='entire_word_checkbutton')
            self.srch_bk_chkbtn = self.rep_dialog.select_single(
                BuilderName='search_backwards_checkbutton')
            self.wrp_chkbtn = self.rep_dialog.select_single(
                BuilderName='wrap_around_checkbutton')
            self.find_button = self.rep_dialog.select_single(
                'GtkLabel', label='_Find')
            self.replace_button = self.rep_dialog.select_single(
                'GtkLabel', label='_Replace')
            self.replace_all_button = self.rep_dialog.select_single(
                'GtkLabel', label='Replace _All')
            self.close_button = self.rep_dialog.select_single(
                'GtkLabel', label='_Close')
            self.search_entry = self.rep_dialog.select_single(
                'GeditHistoryEntry', history_id='search-for-entry')
            self.search_entry_text = self.search_entry.select_single(
                'GtkEntry')
            self.replace_entry = self.rep_dialog.select_single(
                'GeditHistoryEntry', history_id='replace-with-entry')
            self.replace_entry_text = self.replace_entry.select_single(
                'GtkEntry')
            # now lets select the search entry
            self.pointing_device.click_object(self.search_entry)
            # we are all set for the test now
