# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013
#
# Author: Daniel Chapman daniel@chapman-mail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
from autopilot.matchers import Eventually
from testtools import ExpectedException
from testtools.content import Content, ContentType
from testtools.matchers import Equals, NotEquals, Contains, \
    FileExists, FileContains, LessThan, DirExists
from gedit import GeditAutopilotTestCase
from autopilot.testcase import AutopilotTestCase
import subprocess
import testscenarios
import os
import time
import shutil


class GeditKeyboardShortcutTests(GeditAutopilotTestCase):

    def test_close_tab(self, ):
        """ Using shortcut Ctrl+W """
        # setup 2 blank documents
        self.setup_test_docs(2)
        num_before = len(self.app.select_many('GeditView'))
        # close tab
        self.keyboard.press_and_release('Ctrl+w')
        # did it close?
        num_after = len(self.app.select_many('GeditView'))
        self.assertThat(num_after,
                        Equals(num_before - 1),
                        "Tab did not close correctly: %s tabs were still open"
                        % str(num_after))

    def test_save_all_document_shortcut(self, ):
        """ Using shortcut Ctrl+Shift+l, test the save dialog appears """
        # create a 3 docs with some text so we can save
        self.setup_test_docs(3, text=True)
        self.keyboard.press_and_release('Ctrl+s')
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)),
            "Save dialog did not open")

    def test_close_all_tabs_shortcut(self, ):
        """ Using shortcut Ctrl+Shift+W """
        self.setup_test_docs(3)
        # close all windows
        self.keyboard.press_and_release('Ctrl+Shift+w')
        # check there are no tabs
        tabs = len(self.app.select_many('GeditTab'))
        self.assertThat(
            tabs, Equals(0),
            "Not all tabs closed: %s tabs were still open" % str(tabs))

    def test_jump_to_nth_tab_shortcut(self, ):
        """ Using shortcut Alt+n """
        self.setup_test_docs(3)
        # jump to tab 3
        self.keyboard.press_and_release('Alt+3')
        # check we are on tab 3 (index 2)
        self.assertThat(lambda: self.app.select_many('GeditView')[2].is_focus,
                        Eventually(Equals(1)), "Tab 3 did not get focus")

    def test_create_new_document_shortcut(self, ):
        """ Using shortcut Ctrl+N """
        # check that only the default tab is there
        num_tabs = len(self.app.select_many('GeditView'))
        self.assertThat(num_tabs, Equals(1))
        # create new document
        self.keyboard.press_and_release('Ctrl+n')
        # test new document is there and visible
        self.assertThat(num_tabs, Equals(1))
        self.assertThat(lambda: self.app.select_many('GeditView')[1].is_focus,
                        Eventually(Equals(1)), "New document was not created")

    def test_open_file_shortcut(self, ):
        """ Using shortcut Ctrl+o, test the file chooser dialog appears """
        self.keyboard.press_and_release('Ctrl+o')
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "file chooser dialog did not open")

    def test_save_document_shortcut(self, ):
        """ Using shortcut Ctrl+s, test the save dialog appears """
        # create a doc with some text so we can save
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+s')
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "The save dialog did not open")

    def test_save_as_shortcut(self, ):
        """ Using shortcut Ctrl+Shift+s, test the save as dialog appears """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+Shift+s')
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "The save dialog did not open")

    def test_print_document_shortcut(self, ):
        """ Using shortcut Ctrl+p, test the print dialog appears """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+p')
        self.assertThat(
            lambda: self.app.select_single('GtkPrintUnixDialog').visible,
            Eventually(Equals(1)), "The print dialog did not open")

    def test_print_preview_shortcut(self, ):
        """ Using shortcut Ctrl+Shift+p, test the print preview appears """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+Shift+p')
        self.assertThat(
            lambda: self.app.select_single('GeditPrintPreview').visible,
            Eventually(Equals(1)), "The print preview window did not open")

    def test_home_key_moves_to_beginning_of_line(self, ):
        """ Using the 'Home' Key test we go to beginning of line """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Home')
        # lets delete the first letter in the buffer
        self.keyboard.press_and_release('Delete')
        # Lets check we went to beginning and deleted 'D'
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'ocument 1: The quick brown fox jumps over the lazy dog')),
            "Home key did not go to beginning of line")

    def test_end_key_goes_to_end_of_line(self, ):
        """ Using the 'End' Key test we go to end of line """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Home')
        # lets delete the first letter in the buffer
        self.keyboard.press_and_release('Delete')
        # Lets check we went to beginning and deleted 'T'
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'ocument 1: The quick brown fox jumps over the lazy dog')))
        self.keyboard.press_and_release('End')
        self.keyboard.type('s')
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'ocument 1: The quick brown fox jumps over the lazy dogs')
                ),
            "Either the 'End' key did not work or autopilot \
            mis-typed 's': manual compare needed")

    def test_undo_shortcut(self, ):
        """ Using shortcut Ctrl+z, test that we undo last action """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+z')
        # Lets check we were able to undo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy ')),
            "Last word in phrase was not undone")

    def test_redo_shortcut(self, ):
        """Using shortcut Ctrl+Shift+z, test we can redo last undone action"""
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+z')
        # Lets undo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy ')),
            "Last word in phrase was not undone")
        self.keyboard.press_and_release('Ctrl+Shift+z')
        # Lets check we were able to redo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy dog')
                ), "Redo shortcut did not redo the last word in phrase")

    def test_copy_and_paste_shortcuts(self, ):
        """ Using shortcuts Ctrl+c & Ctrl+v test we can copy and paste text """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+a')
        # copy text
        self.keyboard.press_and_release('Ctrl+c')
        self.keyboard.press_and_release('Right')
        # now paste
        self.keyboard.press_and_release('Ctrl+v')
        # did it copy and paste correctly?
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy dog\
                    Document 1: The quick brown fox jumps over the lazy dog'
                )),
            "Three possible reasons for fail: \
            1) Ctrl+a did not select text, \
            2) Ctrl+c did not copy text or \
            3) Ctrl+v did not paste text")

    def test_cut_shortcut(self, ):
        """ Using shortcut Ctrl+x test we can cut text """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+a')
        self.keyboard.press_and_release('Ctrl+x')
        # test it cut the selected text
        self.assertThat(lambda: self.app.select_many('GeditView')[0].buffer,
                        Eventually(Equals('')),
                        "Two possible reasons for fail: \
                        1) Ctrl+a did not select text, \
                        2) Ctrl+x did not cut text.")

    def test_quit_gedit_shortcut(self, ):
        """ Using shortcut Ctrl+q, test gedit quits """
        self.keyboard.press_and_release('Ctrl+q')
        # Since this closes the application and we lose the proxy
        # before we can test its gone, we can expect the exception to
        # assert gedit quit
        with ExpectedException(
                RuntimeError,
                "Application under test exited before the test finished!"):
            gedit = self.app.select_single('GeditWindow')
            self.assertTrue(gedit, Equals(None))

    def test_sidebar_shortcut(self, ):
        """ Using F9 key test side panel opens/closes """
        # test opening panel
        self.keyboard.press_and_release('F9')
        self.assertThat(lambda: self.app.select_many('GeditPanel')[0].visible,
                        Eventually(Equals(1)),
                        "Side panel did not open after pressing F9")
        # and close panel
        self.keyboard.press_and_release('F9')
        self.assertThat(lambda: self.app.select_many('GeditPanel')[0].visible,
                        Eventually(Equals(0)),
                        "Side panel did not close after pressing F9")

    def test_find_a_string_shortcut(self, ):
        """ Using shortcut Ctrl+f test we can search for string """
        # Ctrl+f
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+f')
        self.keyboard.type('fox\n')
        # lets delete the highlighted word
        self.keyboard.press_and_release('Delete')
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown  jumps over the lazy dog')),
            "fox was not highlighted and removed from phrase")

    # TODO:
    # def test_find_next_instance_of_string_shortcut(self, ):
    #    pass
    #
    # def test_find_previous_instance_of_string_shortcut(self, ):
    #    pass

    def test_search_and_replace_shortcut(self, ):
        """ Using shortcut Ctrl+h test the replace dialog opens """
        self.keyboard.press_and_release('Ctrl+h')
        self.assertThat(
            lambda: self.app.select_single('GeditReplaceDialog').visible,
            Eventually(Equals(1)),
            "Replace dialog did not open")
    # TODO:
    # def test_clear_highlighted_text_shortcut(self, ):
    # Ctrl+Shift+k
    #    pass
    #
    # def test_go_to_line_shortcut(self, ):
    # Ctrl+i
    #    pass


class GeditWindowTests(GeditAutopilotTestCase):

    def test_new_document_button(self, ):
        """ Test new doc button creates a new document """
                # check that only the default tab is there
        num_tabs = len(self.app.select_many('GeditView'))
        self.assertThat(num_tabs, Equals(1))
        # create new document
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='FileNew'))
        # test new document is there and visible
        self.assertThat(num_tabs, Equals(1))
        self.assertThat(lambda: self.app.select_many('GeditView')[1].is_focus,
                        Eventually(Equals(1)), "New document was not created")

    def test_open_file_button(self, ):
        """ Using open button, test the file chooser dialog appears """
        self.pointing_device.click_object(
            self.app.select_single('GeditOpenToolButton'))
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "file chooser dialog did not open")

    def test_save_document_button(self, ):
        """ Using save button, test the save dialog appears """
        # create a doc with some text so we can save
        self.setup_test_docs(1, text=True)
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='FileSave'))
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "The save dialog did not open")

    def test_print_document_button(self, ):
        """ Using print button, test the print dialog opens """
        self.setup_test_docs(1)
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='FilePrint'))
        self.assertThat(
            lambda: self.app.select_single('GtkPrintUnixDialog').visible,
            Eventually(Equals(1)), "The print dialog did not open")

    def test_undo_button(self, ):
        """ Test undo button, undoes last action """
        self.setup_test_docs(1, text=True)
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditUndo'))
        # Lets check we were able to undo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy ')),
            "Last word in phrase was not undone")

    def test_redo_button(self, ):
        """ Test redo button, redoes the last undo action """
        self.setup_test_docs(1, text=True)
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditUndo'))
        # Lets check we were able to undo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy ')),
            "Last word in phrase was not undone")
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditRedo'))
        # Lets check we were able to redo the last word
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy dog')
                ), "Redo shortcut did not redo the last word in phrase")

    def test_cut_button(self, ):
        """ Test the cut button cuts the selected text """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+a')
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditCut'))
        # test it cut the selected text
        self.assertThat(lambda: self.app.select_many('GeditView')[0].buffer,
                        Eventually(Equals('')),
                        "Two possible reasons for fail: \
                        1) Ctrl+a did not select text, or \
                        2) Cut button did not cut text.")

    def test_copy_and_paste_buttons(self, ):
        """ test we can copy and paste using buttons """
        self.setup_test_docs(1, text=True)
        self.keyboard.press_and_release('Ctrl+a')
        # copy text
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditCopy'))
        self.keyboard.press_and_release('Right')
        # now paste
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='EditPaste'))
        # did it copy and paste correctly?
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown fox jumps over the lazy dog\
                    Document 1: The quick brown fox jumps over the lazy dog'
                )),
            "Three possible reasons for fail: \
            1) Ctrl+a did not select text, \
            2) Copy button did not work or \
            3) Paste button did not work")

    def test_search_button(self, ):
        """ Test the search button opens the search box """
        self.setup_test_docs(1, text=True)
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='SearchFind'))
        self.keyboard.type('fox\n')
        # lets delete the highlighted word
        self.keyboard.press_and_release('Delete')
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(
                Equals(
                    'Document 1: The quick brown  jumps over the lazy dog')),
            "fox was not highlighted and removed from phrase")

    def test_search_and_replace_button(self, ):
        """ Test the replace button opens the replace dialog """
        self.pointing_device.click_object(
            self.app.select_single('GtkToolButton',
                                   name='SearchReplace'))
        self.assertThat(
            lambda: self.app.select_single('GeditReplaceDialog').visible,
            Eventually(Equals(1)),
            "Replace dialog did not open")

    def test_window_visible(self, ):
        """ Test the window loads """
        self.assertThat(lambda: self.app.select_single('GeditWindow').visible,
                        Eventually(Equals(1)),
                        "Gedit window did not open")

    def test_window_title(self, ):
        """ Test the window title is correct """
        self.assertThat(lambda: self.app.select_single('GeditWindow').title,
                        Eventually(Equals("Untitled Document 1 - gedit")),
                        "Gedit window did not match")

    def test_tab_name(self, ):
        """ Test the tab displays name """
        self.assertThat(lambda: self.app.select_many('GeditTab')[0].name,
                        Eventually(Equals("Untitled Document 1")))

    # CAN WE ACCESS THE BOTTOM COMBOBOX'S????????????


class GeditSidebarTests(GeditAutopilotTestCase):

    def test_close_sidebar_button(self, ):
        # open panel
        self.keyboard.press_and_release('F9')
        self.assertThat(lambda: self.app.select_many('GeditPanel')[0].visible,
                        Eventually(Equals(1)),
                        "Side panel did not open after pressing F9")
        # and close panel using button
        self.pointing_device.click_object(
            self.app.select_many('GeditCloseButton')[0])
        self.assertThat(lambda: self.app.select_many('GeditPanel')[0].visible,
                        Eventually(Equals(0)),
                        "Side panel did not close after clicking close button")

    # TODO: add more maybe test file browser tab and its button objects


class GeditSaveFileTypeTests(GeditAutopilotTestCase):

    scenarios = [
        ('single_char', {'input': "a"}),
        ('short_name', {'input': "test"}),
        ('long_name', {'input': "testfilenameforgedit"}),
        ('extra_long_name', {'input': "autopilotlongfilenamefortesting"}),
        ('python_file', {'input': "gedit.py"}),
        ('C_file', {'input': "gedit.c"}),
        ('cpp_file', {'input': "gedit.cpp"}),
        ('xml_file', {'input': "gedit.xml"}),
        ('vala_file', {'input': "gedit.vala"}),
        ('html_file', {'input': "gedit.html"}),
    ]

    def test_save_file_type_tests(self, ):
        # setup test with empty doc and save dialog opened
        self.setup_test_docs(1, save_dlg=True)
        save_dlg = self.app.select_single('GeditFileChooserDialog')
        self.pointing_device.move_to_object(
            save_dlg.select_single('GtkLabel',
                                   label='_Cancel'))
        # for each scenario enter filename
        self.keyboard.type(self.input)
        # click save

        self.pointing_device.click_object(
            save_dlg.select_single('GtkLabel', label='_Save'))
        # check dialog closed before checking for file
        self.assertThat(lambda: self.app.select_many('GeditTab')[0].name,
                        Eventually(Equals(self.input)),
                        "Tab did not contain filename " + self.input)
        filename = self.input

        self.assertThat('/tmp/' + filename, FileExists())
        # add cleanup
        self.addCleanup(os.remove, '/tmp/%s' % self.input)


class GeditSaveDialogTests(GeditAutopilotTestCase):

    def test_esc_key_closes_dialog(self, ):
        self.setup_test_docs(1, save_dlg=True)
        self.keyboard.press_and_release('Escape')
        # THe fact we can't select dialog shows it closed
        with ExpectedException(AttributeError,
                               "'NoneType' object has no attribute 'visible'"):
            self.assertThat(lambda: self.app.select_single(
                'GeditFileChooserDialog').visible,
                Eventually(Equals(0)),
                "Dialog didn't close")

    def test_cancel_button_closes_dialog(self, ):
        self.setup_test_docs(1, save_dlg=True)
        save_dialog = self.app.select_single('GeditFileChooserDialog')
        self.pointing_device.click_object(
            save_dialog.select_single('GtkLabel',
                                      label='_Cancel'))
        # The fact we can't select dialog shows it closed
        with ExpectedException(AttributeError,
                               "'NoneType' object has no attribute 'visible'"):
            self.assertThat(lambda: self.app.select_single(
                'GeditFileChooserDialog').visible,
                Eventually(Equals(0)),
                "Dialog didn't close")

    def test_new_folder_button(self, ):
        self.setup_test_docs(1, save_dlg=True)
        save_dialog = self.app.select_single('GeditFileChooserDialog')
        self.pointing_device.click_object(
            save_dialog.select_single('GtkButton',
                                      label='Create Fo_lder'))
        self.keyboard.type('test_folder\n')
        self.assertThat('/tmp/test_folder', DirExists())
        self.addCleanup(os.rmdir, '/tmp/test_folder')


class GeditOpenFileTypesTests(GeditAutopilotTestCase):
    scenarios = [
        ('test_file', {'input': "test"}),
        ('py_file', {'input': "gedit.py"}),
        ('c_file', {'input': "gedit1.c"}),
        ('cpp_file', {'input': "gedit.cpp"}),
        ('vala_file', {'input': "gedit.vala"}),
        ('xml_file', {'input': "gedit.xml"}),
    ]

    def test_open_file_type_tests(self, ):
        self.setup_test_docs(1, opn_dlg=True)
        self.keyboard.type(self.input)
        dialog = self.app.select_single('GeditFileChooserDialog')
        self.pointing_device.click_object(dialog.select_single('GtkLabel',
                                                               label='_Open'))
        self.assertThat(lambda: self.app.select_single('GeditWindow').title,
                        Eventually(Contains(self.input)))


class GeditOpenDialogTests(GeditAutopilotTestCase):

    def test_esc_key_closes_dialog(self, ):
        self.pointing_device.click_object(
            self.app.select_single('GeditOpenToolButton'))
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "file chooser dialog did not open")
        self.keyboard.press_and_release('Escape')
        # The fact we can't select dialog shows it closed
        with ExpectedException(AttributeError,
                               "'NoneType' object has no attribute 'visible'"):
            self.assertThat(lambda: self.app.select_single(
                'GeditFileChooserDialog').visible,
                Eventually(Equals(0)),
                "Dialog didn't close")

    def test_cancel_button_closes_dialog(self, ):
        self.pointing_device.click_object(
            self.app.select_single('GeditOpenToolButton'))
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "file chooser dialog did not open")
        open_dialog = self.app.select_single('GeditFileChooserDialog')
        self.pointing_device.click_object(
            open_dialog.select_single('GtkLabel',
                                      label='_Cancel'))
        # The fact we can't select dialog shows it closed
        with ExpectedException(AttributeError,
                               "'NoneType' object has no attribute 'visible'"):
            self.assertThat(lambda: self.app.select_single(
                'GeditFileChooserDialog').visible,
                Eventually(Equals(0)),
                "Dialog didn't close")

    def test_recent_docs_displayed_on_load(self, ):
        self.pointing_device.click_object(
            self.app.select_single('GeditOpenToolButton'))
        self.assertThat(
            lambda: self.app.select_single('GeditFileChooserDialog').visible,
            Eventually(Equals(1)), "file chooser dialog did not open")
        open_dialog = self.app.select_single('GeditFileChooserDialog')
        self.assertThat(lambda: open_dialog.select_single(
            'GtkLabel', label='<b>Recently Used</b>').visible,
            Eventually(Equals(1)), "Recently used not visible")


class GeditReplaceDialogTests(GeditAutopilotTestCase):

    def test_find_and_replace_buttons(self, ):
        self.setup_test_docs(1, text=True, replace_dlg=True)
        self.keyboard.type('fox')
        self.assertTrue(self.search_entry_text.text, Equals('fox'))
        self.pointing_device.click_object(self.replace_entry)
        self.keyboard.type('foxes')
        self.assertTrue(self.replace_entry_text.text, Equals('foxes'))
        self.pointing_device.click_object(self.find_button)
        # lets check replace button is now clickable
        self.assertThat(self.replace_button.sensitive, Eventually(Equals(1)))
        self.pointing_device.click_object(self.replace_button)
        # test text was replaced
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(Equals(
                'Document 1: The quick brown foxes jumps over the lazy dog')))

    def test_match_case_option(self, ):
        # test doesnt match
        self.setup_test_docs(1, text=True, replace_dlg=True)
        self.keyboard.type('Fox')
        self.assertTrue(self.search_entry_text.text, Equals('Fox'))
        self.pointing_device.click_object(self.mtch_chkbtn)
        # check its selected
        self.assertTrue(self.mtch_chkbtn.active, Equals(1))
        self.pointing_device.click_object(self.replace_all_button)
        # FIXME:
        # the replace button should stay inactive as it doesn't match entire
        # word for some reason it says the sensitive property is 1 but in vis
        # shows 0
        # self.pointing_device.click_object(self.find_button)
        # self.assertThat(self.replace_button.sensitive, Equals(0))
        # instead click replace all and check fox was not replaced in the
        # buffer
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(Equals(
                'Document 1: The quick brown fox jumps over the lazy dog')))
        # test does match
        self.pointing_device.click_object(self.search_entry)
        self.keyboard.press_and_release('Ctrl+a')
        self.keyboard.type('fox')
        self.pointing_device.click_object(self.replace_all_button)
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(Equals(
                'Document 1: The quick brown  jumps over the lazy dog')))

    def test_match_entire_word_option(self, ):
        self.setup_test_docs(1, text=True, replace_dlg=True)
        self.keyboard.type('fo')
        self.assertTrue(self.search_entry_text.text, Equals('fo'))
        self.pointing_device.click_object(self.entire_chkbtn)
        # check its selected
        self.assertTrue(self.entire_chkbtn.active, Equals(1))
        # FIXME:
        # the replace button should stay inactive as it doesn't match entire
        # word for some reason it says the sensitive property is 1 but in vis
        # shows 0
        # self.pointing_device.click_object(self.find_button)
        # self.assertThat(self.replace_button.sensitive, Equals(0))
        # instead click replace all and check fox was not replaced in the
        # buffer
        self.pointing_device.click_object(self.replace_all_button)
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(Equals(
                'Document 1: The quick brown fox jumps over the lazy dog')))
        # test does match
        self.pointing_device.click_object(self.search_entry)
        self.keyboard.press_and_release('Ctrl+a')
        self.keyboard.type('fox')
        self.pointing_device.click_object(self.replace_all_button)
        self.assertThat(
            lambda: self.app.select_many('GeditView')[0].buffer,
            Eventually(Equals(
                'Document 1: The quick brown  jumps over the lazy dog')))

    def test_esc_key_closes_dialog(self, ):
        self.setup_test_docs(1, replace_dlg=True)
        self.keyboard.press_and_release('Escape')
        self.assertThat(lambda: self.app.select_single(
            'GeditReplaceDialog').visible,
            Eventually(Equals(0)), "Dialog didn't close")

    def test_close_button_closes_dialog(self, ):
        self.setup_test_docs(1, replace_dlg=True)
        self.pointing_device.click_object(self.close_button)
        self.assertThat(lambda: self.app.select_single(
            'GeditReplaceDialog').visible,
            Eventually(Equals(0)), "Dialog didn't close")


class GeditSingleFileCommandLineTest(AutopilotTestCase):

    def setUp(self):
        super(GeditSingleFileCommandLineTest, self).setUp()
        sample_file = 'test'
        for root, dirs, files in os.walk(os.getcwd()):
            for name in files:
                if name == sample_file:
                    file_path = os.path.abspath(os.path.join(root, name))
        self.gedit = self.launch_test_application("gedit", file_path)

    # can we open files in gedit from terminal
    def test_open_single_file(self, ):
        self.assertThat(lambda: self.gedit.select_single('GeditWindow').title,
                        Eventually(Contains('test')))


class GeditMultiFileCommandLineTest(AutopilotTestCase):

    def setUp(self):
        super(GeditMultiFileCommandLineTest, self).setUp()
        sample_dir = 'samples'
        for root, dirs, files in os.walk(os.getcwd()):
            for name in dirs:
                if name == sample_dir:
                    file_path = os.path.abspath(os.path.join(root, name))
        self.gedit = self.launch_test_application("gedit",
                                                  file_path + '/test',
                                                  file_path + '/gedit.py',
                                                  file_path + '/gedit.c',
                                                  file_path + '/gedit.vala')

    def test_open_multi_files(self, ):

        # the first opened doc 'tab 1' should be in the title
        self.assertThat(lambda: self.gedit.select_single('GeditWindow').title,
                        Eventually(Contains('test')))
        # check 4 tabs are there
        self.assertThat(len(self.gedit.select_many('GeditTab')), Equals(4))
        # TODO: test the contents of the files are diplayed
        # but is this needed? as tab title is covered by
        # another test and so is file contents displayed


class GeditCommandLineTests(AutopilotTestCase):

    def setUp(self):
        super(GeditCommandLineTests, self).setUp()
        self.app = self.launch_test_application("gedit")

    def test_new_doc_arg(self, ):
        # check window open with one tab
        self.assertThat(lambda: self.app.select_single('GeditWindow').visible,
                        Eventually(Equals(1)))
        self.assertThat(len(self.app.select_many('GeditTab')), Equals(1))
        # now lets use the --new-document argument and we should get another
        # tab
        subprocess.check_call(["gedit", "--new-document"])
        self.assertThat(lambda: self.app.select_many(
            'GeditTab')[1].visible, Eventually(Equals(1)))

    def test_new_window_arg(self, ):
        self.assertThat(lambda: self.app.select_single('GeditWindow').visible,
                        Eventually(Equals(1)))
        # now lets use the --new-window argument and we should get another
        # window

        subprocess.check_call(["gedit", "--new-window"])
        self.assertThat(lambda: self.app.select_many('GeditWindow')[1].visible,
                        Eventually(Equals(1)))
        # lets just confirm we have 2 windows
        self.assertThat(len(self.app.select_many('GeditWindow')), Equals(2))
